@@ -1,6 +1,7 @@
 import autograd.numpy as np
 import os
 import copy
+import matplotlib.pyplot as plt
 import pyerrors as pe
 import pytest
 
@@ -56,6 +57,7 @@ def test_Obs_exceptions():
     one.gamma_method()
     with pytest.raises(Exception):
         one.plot_piechart()
+    plt.close('all')
 
 def test_dump():
     value = np.random.normal(5, 10)
@@ -368,6 +370,7 @@ def test_utils():
         assert my_obs < (my_obs + 1)
         float(my_obs)
         str(my_obs)
+        plt.close('all')
 
 
 def test_cobs():
@@ -855,7 +858,7 @@ def test_covariance_additional_non_overlapping_data():
     assert np.isclose(corr1[0, 1], corr2[0, 1], atol=1e-14)
 
 
-def test_coavariance_reorder_non_overlapping_data():
+def test_covariance_reorder_non_overlapping_data():
     range1 = range(1, 20, 2)
     range2 = range(1, 41, 2)
 
@@ -881,6 +884,12 @@ def test_empty_obs():
     assert q == o
 
 
+def test_reweight_method():
+    obs1 = pe.pseudo_Obs(0.2, 0.01, 'test')
+    rw = pe.pseudo_Obs(0.999, 0.001, 'test')
+    assert obs1.reweight(rw) == pe.reweight(rw, [obs1])[0]
+
+
 def test_jackknife():
     full_data = np.random.normal(1.1, 0.87, 5487)
 
@@ -941,3 +950,16 @@ def test_cobs_array():
     cobs * np.identity(4)
     np.identity(4) / cobs
     cobs / np.ones((4, 4))
+
+
+def test_hash():
+    obs = pe.pseudo_Obs(0.3, 0.1, "test") + pe.Obs([np.random.normal(2.3, 0.2, 200)], ["test2"], [range(1, 400, 2)])
+    o1 = obs + pe.cov_Obs(0.0, 0.1, "co") + pe.cov_Obs(0.0, 0.8, "co2")
+    o2 = obs + pe.cov_Obs(0.0, 0.2, "co") + pe.cov_Obs(0.0, 0.8, "co2")
+
+    for i_obs in [obs, o1, o2]:
+        assert hash(i_obs) == hash(i_obs ** 2 / i_obs) == hash(1 * i_obs)
+        assert hash(i_obs) == hash((1 + 1e-16) * i_obs)
+        assert hash(i_obs) != hash((1 + 1e-7) * i_obs)
+    assert hash(obs) != hash(o1)
+    assert hash(o1) != hash(o2)
